from functools import wraps
from typing import Optional

from traceloop.semconv import SpanAttributes, TraceloopSpanKindValues
from traceloop.tracing.tracer import Tracer


def task(name: Optional[str] = None, tlp_span_kind: Optional[TraceloopSpanKindValues] = TraceloopSpanKindValues.TASK):
    def decorate(fn):
        @wraps(fn)
        def wrap(*args, **kwargs):
            span_name = f"{name}.{tlp_span_kind.value}" if name else f"{fn.__name__}.{tlp_span_kind.value}"
            with Tracer.instance().start_as_current_span(span_name) as span:
                span.set_attribute(SpanAttributes.TRACELOOP_SPAN_KIND, tlp_span_kind.value)
                return fn(*args, **kwargs)

        return wrap

    return decorate


def workflow(name: Optional[str] = None):
    def decorate(fn):
        @wraps(fn)
        def wrap(*args, **kwargs):
            span_name = f"{name}.workflow" if name else f"{fn.__name__}.workflow"
            with Tracer.instance().start_as_current_span(span_name) as span:
                span.set_attribute(SpanAttributes.TRACELOOP_SPAN_KIND, TraceloopSpanKindValues.WORKFLOW.value)
                return fn(*args, **kwargs)

        return wrap

    return decorate


def agent(name: Optional[str] = None):
    return task(name=name, tlp_span_kind=TraceloopSpanKindValues.AGENT)


def tool(name: Optional[str] = None):
    return task(name=name, tlp_span_kind=TraceloopSpanKindValues.TOOL)



import os

from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.http.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from traceloop.instrumentation.openai import OpenAIInstrumentor
import importlib.util

TRACER_NAME = "traceloop.tracer"
TRACELOOP_API_ENDPOINT = "https://api.traceloop.dev/v1/traces"


class Tracer:
    __instance = None

    @staticmethod
    def init():
        api_key = os.getenv("TRACELOOP_API_KEY")
        api_endpoint = os.getenv("TRACELOOP_API_ENDPOINT") or TRACELOOP_API_ENDPOINT
        print(f"Initializing Traceloop Tracer... API endpoint: {api_endpoint}")
        provider = TracerProvider()
        exporter = OTLPSpanExporter(
            endpoint=api_endpoint,
            headers={
                "Authorization": f"Bearer {api_key}",
            }
        )
        processor = BatchSpanProcessor(exporter)
        provider.add_span_processor(processor)
        trace.set_tracer_provider(provider)
        Tracer.__instance = trace.get_tracer(TRACER_NAME)

        # os.environ["OTEL_PYTHON_REQUESTS_EXCLUDED_URLS"] = "/openai/"

        if importlib.util.find_spec("openai") is not None:
            OpenAIInstrumentor().instrument()
        else:
            raise Exception("OpenAI sdk is not installed")

        if importlib.util.find_spec("requests") is not None:
            from opentelemetry.instrumentation.requests import RequestsInstrumentor
            RequestsInstrumentor().instrument(excluded_urls="^https://api.openai.com")

        if importlib.util.find_spec("mysql") is not None:
            from opentelemetry.instrumentation.mysql import MySQLInstrumentor
            MySQLInstrumentor().instrument()

    @staticmethod
    def instance():
        if Tracer.__instance is None:
            raise Exception("Tracer is not initialized")
        return Tracer.__instance
